#include "stdafx.h"
#include "FileTreeView.h"
#include "../resource.h"
#include "../Dialogs/LogDialog.h"


CFileTreeView::CFileTreeView(CFileListModel &rModel):
	m_bSelfAction(false),
	m_hFrameWnd(NULL),
	m_DropTarget(this),
	m_hDropHilight(NULL),
	m_rModel(rModel)
{
}

LRESULT CFileTreeView::OnCreate(LPCREATESTRUCT lpcs)
{
	LRESULT lRes=DefWindowProc();
	SetFont(AtlGetDefaultGuiFont());

	//fɃCxgnho^
	m_rModel.addEventListener(m_hWnd);

	//C[WXg쐬
	m_ImageList.Create(::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CXSMICON), ILC_COLOR32 | ILC_MASK, 8, 1);
	SetImageList(m_ImageList,TVSIL_NORMAL);
	//tH_ACR擾
	SHFILEINFO shfi;
	SHGetFileInfo(_T("dummy"),FILE_ATTRIBUTE_DIRECTORY,&shfi,sizeof(shfi),SHGFI_USEFILEATTRIBUTES|SHGFI_ICON|SHGFI_SMALLICON);
	m_ImageList.AddIcon(shfi.hIcon);
	DestroyIcon(shfi.hIcon);
	SHGetFileInfo(_T("dummy"),FILE_ATTRIBUTE_DIRECTORY,&shfi,sizeof(shfi),SHGFI_USEFILEATTRIBUTES|SHGFI_ICON|SHGFI_SMALLICON|SHGFI_OPENICON);
	m_ImageList.AddIcon(shfi.hIcon);
	DestroyIcon(shfi.hIcon);

	return lRes;
}

LRESULT CFileTreeView::OnDestroy()
{
	m_rModel.removeEventListener(m_hWnd);
	m_ImageList.Destroy();
	return 0;
}

LRESULT CFileTreeView::OnFileListArchiveLoaded(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	Clear();
	ConstructTree();

	//t@Chbv̎󂯓
	EnableDropTarget(m_rModel.IsAddItemsSupported());
	return 0;
}


LRESULT CFileTreeView::OnFileListNewContent(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	UpdateCurrentNode();
	return 0;
}

LRESULT CFileTreeView::OnFileListUpdated(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	//nothing to do
	return 0;
}

bool CFileTreeView::UpdateCurrentNode()
{
	ARCHIVE_ENTRY_INFO_TREE* lpNode=m_rModel.GetCurrentNode();
	ITEMDICT::iterator ite=m_TreeItemMap.find(lpNode);
	if(m_TreeItemMap.end()==ite){
		return false;
	}
	EnsureVisible((*ite).second);
	SelectItem((*ite).second);
	return true;
}


bool CFileTreeView::ConstructTree(HTREEITEM hParentItem,ARCHIVE_ENTRY_INFO_TREE* lpNode)
{
	BeginSelfAction();
	HTREEITEM hItem;
	if(NULL==hParentItem){
		lpNode=m_rModel.GetRootNode();
		hItem=InsertItem(PathFindFileName(m_rModel.GetArchiveFileName()), TVI_ROOT, TVI_LAST);
		SetItemImage(hItem,2,2);

		//A[JCuACR̐ݒ
		m_ImageList.Remove(2);	//ÂACR폜

		//֘At̃AACR擾
		SHFILEINFO shfi;
		::SHGetFileInfo(m_rModel.GetArchiveFileName(), 0, &shfi, sizeof(shfi),SHGFI_ICON | SHGFI_SMALLICON);
		m_ImageList.AddIcon(shfi.hIcon);
	}else{
		//qǉ
		hItem=InsertItem(lpNode->strTitle,hParentItem,TVI_LAST);
		SetItemImage(hItem,0,1);
	}
	m_TreeItemMap.insert(ITEMDICT::value_type(lpNode,hItem));
	//ACẽf[^Ƀm[h̃|C^ݒ
	SetItemData(hItem,(DWORD_PTR)lpNode);

	//*Nodez̃tH_ɑ΂ď
	UINT numItems=lpNode->GetNumChildren();
	for(UINT i=0;i<numItems;i++){
		ARCHIVE_ENTRY_INFO_TREE* lpChild=lpNode->GetChild(i);
		if(lpChild->bDir){
			//fBNgȂǉ
			ConstructTree(hItem,lpChild);
		}
	}
	return true;
}

void CFileTreeView::Clear()
{
	DeleteAllItems();
	m_TreeItemMap.clear();
}

void CFileTreeView::ExpandTree()
{
	ITEMDICT::iterator ite=m_TreeItemMap.begin();
	ITEMDICT::iterator end=m_TreeItemMap.end();
	for(;ite!=end;++ite){
		Expand((*ite).second);
	}
}

LRESULT CFileTreeView::OnTreeSelect(LPNMHDR)
{
	TRACE(__FUNCTIONW__ _T("\n"));
	if(IsSelfAction()){
		//ConstructTree()ɂ莩I񂾎AŔĂ܂Ȃ߂̃tO
		EndSelfAction();
	}else{
		//IꂽACeɊ֘Atꂽm[h擾A
		//̔z̃t@CXgr[ɕ\
		ARCHIVE_ENTRY_INFO_TREE* lpCurrent=m_rModel.GetCurrentNode();

		HTREEITEM hItem=GetSelectedItem();
		if(!hItem)return 0;

		ARCHIVE_ENTRY_INFO_TREE* lpNode=(ARCHIVE_ENTRY_INFO_TREE*)GetItemData(hItem);
		ASSERT(lpNode);

		//TODO
		//if(!m_FileListModel.IsRoot())m_FileListModel.GetRootNode().EndFindItem();
		if(lpNode && lpNode!=lpCurrent){
			m_rModel.SetCurrentNode(lpNode);
		}
	}
	return 0;
}


void CFileTreeView::EnableDropTarget(bool bEnable)
{
	if(bEnable){
		//hbv󂯓ݒ
		::RegisterDragDrop(m_hWnd,&m_DropTarget);
	}else{
		//hbv󂯓Ȃ
		::RevokeDragDrop(m_hWnd);
	}
}

//---------------------------------------------------------
//    IDropCommunicator̎:hbO&hbvɂ鈳k
//---------------------------------------------------------
HRESULT CFileTreeView::DragEnter(IDataObject *lpDataObject,POINTL &pt,DWORD &dwEffect)
{
	return DragOver(lpDataObject,pt,dwEffect);
}

HRESULT CFileTreeView::DragLeave()
{
	//SẴnCCg𖳌
	if(m_hDropHilight){
		SetItemState( m_hDropHilight, ~LVIS_DROPHILITED, LVIS_DROPHILITED);
	}
	m_hDropHilight=NULL;
	return S_OK;
}

HRESULT CFileTreeView::DragOver(IDataObject *,POINTL &pt,DWORD &dwEffect)
{
	//tH[}bgɑΉ
	if(!m_DropTarget.QueryFormat(CF_HDROP)){	//t@Cp
		//t@Cł͂Ȃ̂ŋ
		dwEffect = DROPEFFECT_NONE;
	}else{
		//---hbvACe擾
		CPoint ptTemp(pt.x,pt.y);
		ScreenToClient(&ptTemp);
		HTREEITEM hItem=HitTest(ptTemp,NULL);

		//---邽߁AOƓACenCCgȂnCCgNAȂ
		if(hItem!=m_hDropHilight){
			//ÕnCCg𖳌
			if(m_hDropHilight){
				SetItemState( m_hDropHilight, ~LVIS_DROPHILITED, LVIS_DROPHILITED);
			}
			//VnCCg
			if(hItem){
				SelectDropTarget(hItem);
				m_hDropHilight=hItem;
			}
		}
		dwEffect = hItem ? DROPEFFECT_COPY : DROPEFFECT_NONE;
	}
	return S_OK;
}

//t@C̃hbv
HRESULT CFileTreeView::Drop(IDataObject *lpDataObject,POINTL &pt,DWORD &dwEffect)
{
	//ÕnCCg𖳌
	if(m_hDropHilight){
		SetItemState( m_hDropHilight, ~LVIS_DROPHILITED, LVIS_DROPHILITED);
	}else{
		return E_HANDLE;
	}

	//t@C擾
	std::list<CString> fileList;
	if(S_OK==m_DropTarget.GetDroppedFiles(lpDataObject,fileList)){
		//t@Chbvꂽ
		dwEffect = DROPEFFECT_COPY;

		//---hbv
		CPoint ptTemp(pt.x,pt.y);
		ScreenToClient(&ptTemp);
		HTREEITEM hItem=HitTest(ptTemp,NULL);

		CString strDest;	//荞ސ
		ARCHIVE_ENTRY_INFO_TREE* lpNode=(ARCHIVE_ENTRY_INFO_TREE*)GetItemData(hItem);
		if(lpNode){		//ACeDnD
			//ACetH_炻̃tH_ɒǉ
			ASSERT(lpNode->bDir);
			ArcEntryInfoTree_GetNodePathRelative(lpNode,m_rModel.GetRootNode(),strDest);
		}else{
			return E_HANDLE;
		}
		TRACE(_T("Target:%s\n"),(LPCTSTR)strDest);

		//ǉJn
		::EnableWindow(m_hFrameWnd,FALSE);
		CString strLog;

		HRESULT hr=m_rModel.AddItem(fileList,strDest,strLog);

		::EnableWindow(m_hFrameWnd,TRUE);
		::SetForegroundWindow(m_hFrameWnd);

		if(FAILED(hr) || S_FALSE==hr){
			CString msg;
			switch(hr){
			case E_LF_SAME_INPUT_AND_OUTPUT:	//A[JCugǉ悤Ƃ
				msg.Format(IDS_ERROR_SAME_INPUT_AND_OUTPUT,m_rModel.GetArchiveFileName());
				ErrorMessage(msg);
				break;
			case E_LF_UNICODE_NOT_SUPPORTED:	//t@CUNICODEt@Ck悤Ƃ
				ErrorMessage(CString(MAKEINTRESOURCE(IDS_ERROR_UNICODEPATH)));
				break;
			case S_FALSE:	//ǉɖ
				{
					CLogDialog LogDialog;
					LogDialog.SetData(strLog);
					LogDialog.DoModal();
				}
				break;
			default:
				ASSERT(!"This code cannot be run");
			}
			return E_INVALIDARG;
		}
		//A[JCuXV
		::PostMessage(m_hFrameWnd,WM_FILELIST_REFRESH,0,0);
		return S_OK;
	}else{
		//󂯓łȂ`
		dwEffect = DROPEFFECT_NONE;
		return S_FALSE;	//S_OK
	}
}

